@@ -39,7 +39,7 @@ MainWindow::initLogging ()
                         this     , SLOT(onCleanupLogsArray()));
     Q_ASSERT(rv); Q_UNUSED(rv);
 
-    Q_DEBUG("Using qgvdial version 0.3.1021");
+    Q_DEBUG("Using qgvdial version 0.3.1032");
 }//MainWindow::initLogging
 
 /** Log information to console and to log file
@@ -108,9 +108,9 @@ MainWindow::setStatus(const QString &strText, int timeout /* = 3000*/)
             theLabel = new QLabel (strText, &infoBox);
             theLabel->setAlignment (Qt::AlignHCenter);
             infoBox.setWidget (theLabel);
-            qDebug("Created the Maemo5 yellow banner label");
+            Q_DEBUG("Created the Maemo5 yellow banner label");
         } else {
-            qDebug() << "Display the status banner:" << strText;
+            Q_DEBUG(QString("Display the status banner: %1").arg (strText));
             theLabel->setText (strText);
         }
         infoBox.setTimeout (0 == timeout ? 3000 : timeout);
@@ -219,7 +219,7 @@ MainWindow::onGetLogLocation(bool success, const QByteArray &response,
         QDomElement appVerTag = doc.createElement("Version");
         paramsTag.appendChild(appVerTag);
 
-        QDomText appVerText = doc.createTextNode("0.3.1021");
+        QDomText appVerText = doc.createTextNode("0.3.1032");
         appVerTag.appendChild(appVerText);
 
         QDomElement osVerTag = doc.createElement("OsVer");
@@ -287,7 +287,7 @@ MainWindow::onLogPosted(bool success, const QByteArray &response,
         QDateTime dt = token->inParams["date"].toDateTime();
         QString body = QString("Logs captured at %1 \n")
                                 .arg (dt.toUTC ().toString (Qt::ISODate));
-        body += "qgvdial version = 0.3.1021 \n";
+        body += "qgvdial version = 0.3.1032 \n";
         body += QString("OS: %1 \n").arg(osd.getOSDetails());
         body += QString("Logs are in %1 \n").arg(strReply);
         url.addQueryItem ("body", body);
