@@ -216,7 +216,7 @@ private:
     bool connectSettingsSignals();
 
 
-    void initContacts ();
+    void initContacts (const QString &contactsPass);
     void deinitContacts ();
 
     void initInbox ();
@@ -258,10 +258,11 @@ private:
     QIcon           icoQgv;
     QSystemTrayIcon *pSystray;
     //! GV Contacts object
-    GVContactsTable oContacts;
+    GVContactsTable *oContacts;
     //! GV Inbox object
-    GVInbox         oInbox;
+    GVInbox         *oInbox;
 
+    bool bBeginPlayAfterLoad;
     //! The Phonon vmail player
     Phonon::MediaObject *vmailPlayer;
 
@@ -290,7 +291,7 @@ private:
     QString         strPass;
 
     //! Model for registered phone numbers
-    RegNumberModel  modelRegNumber;
+    RegNumberModel *modelRegNumber;
     //! Index of the registered phone currently in use
     int             indRegPhone;
 
